package com.tdws.pandora.exceptions;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.Writer;
import java.net.URL;

import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.jboss.logging.Logger;
import org.jboss.soa.esb.message.Message;
import org.xml.sax.SAXException;

public class ValidationUtil {
	
	private Logger logger = Logger.getLogger(ValidationUtil.class);
	String xsdName;
	
	public static boolean validateMessage(Message message,String name) throws TransformerException, SAXException, IOException
	{  
		
		ValidationUtil vUtil = new ValidationUtil();
		vUtil.xsdName = name;
		
		InputStream inputMessage = vUtil.createInputStream(message);
		Writer outWriter = new StringWriter();
		StreamResult result = new StreamResult( outWriter );
		vUtil.transformMessage(inputMessage,result);
		String truncatedMessage = outWriter.toString();
		Boolean bResult = vUtil.validate(truncatedMessage);
		
		return bResult;
		
	}
	
	
	private InputStream createInputStream(Message message)
	{
		logger.info("Entering createInputStream Method");
		String str = message.getBody().get().toString();
		InputStream is = new ByteArrayInputStream(str.getBytes());
		
		logger.info("Exiting createInputStream Method");
		return is;
	}

	private void transformMessage(InputStream isMessage, StreamResult res) throws TransformerException
	{
		logger.info("Entering transformMessage Method");
		TransformerFactory factory = TransformerFactory.newInstance();
		InputStream inputStream = Thread.currentThread().getContextClassLoader().getResourceAsStream("/META-INF/NEW_XSL.xsl");
		StreamSource xslStream = new StreamSource(inputStream);
		Transformer transformer = factory.newTransformer(xslStream);
		StreamSource in = new StreamSource(isMessage);
		transformer.transform(in, res);
		logger.info("Exiting transformMessage Method");
	}

	private boolean validate(String inputXml)throws SAXException,IOException 
	{
		logger.info("Entering validate Method");
		SchemaFactory factory = SchemaFactory.newInstance("http://www.w3.org/2001/XMLSchema");
		URL url = Thread.currentThread().getContextClassLoader().getResource("/META-INF/wsdl/MobileDeviceAccountManagement/" + xsdName);
		Schema schema = factory.newSchema(url);
		Validator validator = schema.newValidator();
		Source source = new StreamSource(new StringReader(inputXml));
		boolean isValid = true;
		try {
			validator.validate(source);
		} catch (SAXException e) {
			logger.error("Error in Validation");
			System.err.println("Not valid");
			isValid = false;
			throw e;
			
		}
		logger.info("Exiting validate Method");
		return isValid;
	}


}
