package com.tdws.pandora.utils;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.Writer;
import java.net.URL;
import java.util.Map;

import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.jboss.logging.Logger;
import org.jboss.soa.esb.message.Message;
import org.xml.sax.SAXException;

import com.tdws.pandora.constants.PandoraConstants;

public class ValidationUtil {
	
	private static final Logger logger = Logger.getLogger(ValidationUtil.class);
	String xsdPath;
	String xslPath;
	/**
	 * this method is called for request and response valodation
	 * @param message
	 * @param xsdPath
	 * @return
	 * @throws TransformerException
	 * @throws SAXException
	 * @throws IOException
	 */
	public static boolean validateMessage(Message message,Map<String,String> validatorConfig) throws TransformerException, SAXException, IOException {  
		ValidationUtil vUtil = new ValidationUtil();
		vUtil.xsdPath = validatorConfig.get(PandoraConstants.XSD_PATH);
		vUtil.xslPath = validatorConfig.get(PandoraConstants.XSL_PATH);
		
		InputStream inputMessage = vUtil.createInputStream(message);
		Writer outWriter = new StringWriter();
		StreamResult result = new StreamResult( outWriter );
		vUtil.transformMessage(inputMessage,result);
		String truncatedMessage = outWriter.toString();
		Boolean bResult = vUtil.validate(truncatedMessage);
		
		return bResult;
		
	}
	
	/**this method is used to create an InputStream from the Message
	 * @param message
	 * @return
	 */
	private InputStream createInputStream(Message message) 	{
		logger.debug("Entering createInputStream Method");
		//String str  = new String((byte[])message.getBody().get());
		String str = message.getBody().get().toString();
		InputStream is = new ByteArrayInputStream(str.getBytes());
		
		logger.debug("Exiting createInputStream Method");
		return is;
	}

	/**this method is used to get the body of the Message using XSLT
	 * @param inputStream
	 * @param streamResult
	 * @throws TransformerException
	 */
	private void transformMessage(InputStream inputStream, StreamResult streamResult) throws TransformerException 	{
		logger.debug("Entering transformMessage Method");
		TransformerFactory factory = TransformerFactory.newInstance();
		InputStream inputStreamXsl = Thread.currentThread().getContextClassLoader().getResourceAsStream(xslPath);
		StreamSource xslStream = new StreamSource(inputStreamXsl);
		Transformer transformer = factory.newTransformer(xslStream);
		StreamSource in = new StreamSource(inputStream);
		transformer.transform(in, streamResult);
		logger.debug("Exiting transformMessage Method");
	}

	/**
	 * @param inputXml
	 * @return
	 * @throws SAXException
	 * @throws IOException
	 */
	private boolean validate(String inputXml)throws SAXException,IOException 
	{
		logger.debug("Entering validate Method");
		SchemaFactory factory = SchemaFactory.newInstance(PandoraConstants.SCHEMA); //Move it to Classlevel Constants
		URL url = Thread.currentThread().getContextClassLoader().getResource(xsdPath);
		Schema schema = factory.newSchema(url);
		Validator validator = schema.newValidator();
		Source source = new StreamSource(new StringReader(inputXml));
		boolean isValid = true;
		try {
			validator.validate(source);
		} catch (SAXException e) {
			logger.error("Error in Validation");
			System.err.println("Not valid");
			isValid = false;
			throw e;
			
		}
		logger.debug("Exiting validate Method");
		return isValid;
	}


}
